import { MatchResult } from 'path-to-regexp';

type Middleware = (params: RequestParams, handlerFn?: Handler) => Response | Middleware | Promise<Middleware | Response | undefined>;

/**
 * Handles incoming Cloudflare Worker requests
 */
declare const handleEntryRequest: (request: Request, env: any) => Promise<Response | Middleware>;
declare const readRequestBody: (request: any) => Promise<any>;

interface PutRequestObject {
    [propertyName: string]: any;
}
interface ListRequestObject {
    prefix?: string;
    reverse?: boolean;
    limit?: number | null;
    noCache?: boolean;
}
type StateFn = (objectName?: string, objectId?: string) => StateMethods;
interface StateMethods {
    get: (propertyName: string) => Promise<any>;
    put: (putRequestObject: PutRequestObject) => Promise<any>;
    delete: (propertyName: string) => Promise<any>;
    deleteAll: () => Promise<any>;
    list: (listRequestObject?: ListRequestObject) => Promise<any>;
}

interface RequestParams {
    state: StateFn;
    matches: MatchResult<any>;
    body: any;
    headers: Headers;
    request: Request;
}
type Handler = ((params: RequestParams) => Response | Promise<Response>);
interface RouteObject {
    [route: string]: Handler;
}

interface Routes {
    [route: string]: any;
}
interface Controllers {
    [className: string]: Controller;
}
interface Options {
    routes?: Routes;
    controllers?: Controllers;
    debug?: boolean;
    objectVersion?: string;
    exports?: any;
    objects?: string[];
    objectStateMapping?: ObjectStateMapping;
    env?: any;
    authRoutes?: boolean;
    requestParams?: RequestParams;
    firewall?: Firewall | boolean;
    adminPanel?: boolean;
    name?: string;
    email?: EmailOptions;
}
type Timings = {
    [timingName: string]: number;
};
type ObjectStateMapping = {
    [objectName: string]: string;
};
interface EmailOptions {
    name?: string;
    senderEmail: string;
}
interface Firewall {
    limitRequestsPerMinute: number;
}
type Controller = new (...args: any[]) => any;

declare class ResponseParams {
    errors: string[];
    /**
     * Sets an error response
     */
    setError: (errorMessage: string) => void;
    /**
     * Gets the last error response
     */
    getLastError: () => string | null;
}

declare class Apiker {
    name: string;
    routes: Routes;
    controllers: Controllers;
    debug: boolean;
    objectVersion: string;
    objects: string[];
    objectStateMapping: ObjectStateMapping;
    authRoutes: boolean;
    env: any;
    requestParams: RequestParams;
    responseParams: ResponseParams;
    responseHeaders: Headers;
    firewall: Firewall | boolean;
    adminPanel: boolean;
    email?: EmailOptions;
    timings: Timings;
    defaultObjectName: string;
    /**
     * Initialize Apiker, perform basic validation
     */
    init: (options?: Options) => Response | undefined;
    /**
     * Set options
     */
    setProps: (options?: Options) => void;
    /**
     * Creates a durable object class definition
     */
    getObjectClassDefinition: (objectName: string) => {
        new (state: any): {
            state: any;
            fetch: (request: any) => Promise<Response>;
        };
    };
}
declare const apiker: Apiker;
//# sourceMappingURL=Apiker.d.ts.map

/**
 * Responses
 */
declare const getAuthRoutes: () => RouteObject;

interface User {
    id: string;
    email: string;
    role?: string;
    password: string;
    verified: boolean;
    createdAt: number;
    updatedAt: number;
}

/**
 * Encodes an input into Base64
 * @param inputStr Base64 input to decode
 * @returns
 */
declare const encodeString: (inputStr: any) => any;
/**
 * Decodes a Base64 input
 * @param inputStr Base64 input to decode
 * @returns
 */
declare const decodeString: (inputStr: any) => any;
/**
 * Generates a JWT token
 * @param data Payload to include in the token
 * @param expirationInMinutes Expiration of the token (in minutes)
 * @returns
 */
declare const createJWT: (data: any, expirationInMinutes?: number) => string;
/**
 * Parses a JWT token
 * @param token JWT token
 * @returns
 */
declare const parseJWT: (token: string, disableClientIdCheck?: boolean) => any;
/**
 * Generates a bcrypt hash of a given message
 * @param message Message to hash
 * @returns
 */
declare const hash_bcrypt: (message: string) => string;
/**
 * Compares a raw value with a bcrypt hash
 * @param message Raw input to compare
 * @param hash Hash to compare against
 * @returns
 */
declare const compare_bcrypt: (message: string, hash: string) => boolean;
/**
 * Creates a Base64 hash signed with the Apiker's installation secret key
 */
declare const sign: (message: any) => string;
/**
 * Creates a SHA256 hash signed with the Apiker's installation secret key
 */
declare const sign_sha256: (message: string) => string;
/**
 * Creates a SHA1 hash signed with the Apiker's installation secret key
 */
declare const sign_sha1: (message: string) => string;
/**
 * Creates a random SHA256 hash
 */
declare const randomHash: () => any;
/**
 * Creates a random SHA1 hash
 */
declare const randomHash_SHA1: () => string;
/**
 * Generating a clientId
 * This value is returned to client and not stored
 */
declare const getClientId: () => string;
/**
 * Retrieves the auth tokens from the request headers or cookies.
 */
declare const extractToken: () => string;
/**
 * Fetches the current user by checking for the auth token in the request headers or in the cookies.
 */
declare const getCurrentUser: (disableClientIdCheck?: boolean) => Promise<User | undefined>;
/**
 * Checks whether a given user is an admin
 */
declare const isUserAdmin: (userId?: string) => Promise<boolean>;
/**
 * Checks whether the current user is an admin
 */
declare const isCurrentUserAdmin: () => Promise<boolean>;
/**
 * Generates auth tokens for a given user
 * @param userId The user's ID
 * @param expirationInMinutes The expiration time for the action token. The refresh token does not expire.
 * @returns
 */
declare const getTokens: (userId: string, expirationInMinutes?: number) => {
    userId: string;
    token: string;
    refreshToken: string;
};
/**
 * Gets the client's IP
 */
declare const getRawIp: () => string;
/**
 * Gets the client's IP signed in SHA1 format.
 */
declare const getSignedIp: () => string;

declare const loginUser: Handler;
declare const loginUserAction: (email: string, password: string) => Promise<{
    userId: string;
    token: string;
    refreshToken: string;
    email: string;
} | undefined>;
declare const checkUser: (email: string, password: string) => Promise<User | undefined>;

declare const refreshUser: Handler;

declare const registerUser: Handler;
declare const registerUserAction: (email: string, password: string, extraParams?: any) => Promise<User | undefined>;

declare const forgotUser: Handler;
declare const forgotUserAction: (email: string) => Promise<Response | undefined>;
/**
 * Forgot user reset action
 */
declare const forgotUserReset: Handler;

declare const OBN: {
    COMMON: string;
    USERS: string;
    EMAILTOUUID: string;
    RATELIMIT: string;
    LOGS: string;
    BANS: string;
};
declare const OBMT: {
    DEFAULT: string;
    SIGNEDIP: string;
    CLIENTID: string;
    IP: string;
};

declare const res_200: (input?: any, options?: null) => Response;
declare const res_204: (input?: any, options?: null) => Response;
declare const res_400: (input?: any, options?: null) => Response;
declare const res_401: (input?: any, options?: null) => Response;
declare const res_404: (input?: any, options?: null) => Response;
declare const res_429: (input?: any, options?: null) => Response;
declare const res_500: (input?: any, options?: null) => Response;
declare const res: (input: any, options?: any) => Response;
declare const resRaw: (htmlContent: string, contentType?: string) => Response;

declare const RESPONSE_HEADERS_DEFAULT: {
    "Access-Control-Allow-Origin": string;
    "Access-Control-Allow-Headers": string;
    "Access-Control-Allow-Credentials": string;
    "Access-Control-Allow-Methods": string;
    "content-type": string;
};
declare const RESPONSE_MESSAGES: {
    200: string;
    204: string;
    400: string;
    401: string;
    404: string;
    429: string;
    500: string;
};

declare const rateLimitRequest: (prefix: string, params: RequestParams, handlerFn?: Handler, limit?: number, timeLapse?: number, onLimitReached?: any) => Promise<any>;
declare const isRateLimitReached: (prefix: string, limit: number, timeLapse: number) => Promise<{
    rateLimitReached: boolean | 0;
    requestCount: number;
}>;

interface LogObject {
    propertyName: string;
    time: number;
    id: string;
    clientId: string;
    countryCode: string;
    pathname: string;
    issuedBy?: string;
}

declare const addUniqueLogEntry: (prefix: string, additionalParams?: {}, objectName?: string, signedIp?: string | undefined, clientId?: string | undefined) => Promise<void>;
declare const addLogEntry: (prefix: string, additionalParams?: any, objectName?: string, signedIp?: string | undefined, clientId?: string | undefined) => Promise<void>;
declare const getLogParams: (propertyName: string, signedIp?: string | undefined, clientId?: string | undefined, additionalParams?: {}) => LogObject;
declare const getUserLogPropertyName: (prefix: string, signedIp?: string | null) => string;
declare const getUserLogEntries: (prefix: string, limit?: number, objectName?: string, signedIp?: string | undefined) => Promise<LogObject[]>;
declare const getLogEntries: (prefix: string, limit?: number | null, objectName?: string) => Promise<LogObject[]>;
declare const deleteAllLogsInObject: (objectName: string, signedIp: string) => Promise<any>;
declare const deleteUserLogEntries: (prefix: string, objectName?: string, signedIp?: string | undefined) => Promise<any[]>;
declare const deleteLogEntries: (prefix: string, objectName?: string) => Promise<any[]>;
declare const getAllLogEntries: (objectName?: string, limit?: number | null, objectId?: string) => Promise<LogObject[]>;

declare const measureTiming: (timingName: string) => number;
declare const elapsedSinceRequestStart: () => number;

interface EmailActor {
    name: string;
    email: string;
}

declare const sendEmail: (subject: string, content: string, to?: EmailActor[], sender?: EmailActor) => Promise<Response | undefined>;

declare const EMAIL_API_ENDPOINT = "https://api.brevo.com/v3/smtp/email";

export { Controller, Controllers, EMAIL_API_ENDPOINT, EmailOptions, Firewall, Handler, LogObject, OBMT, OBN, ObjectStateMapping, Options, RESPONSE_HEADERS_DEFAULT, RESPONSE_MESSAGES, RequestParams, ResponseParams, RouteObject, Routes, Timings, User, addLogEntry, addUniqueLogEntry, apiker, checkUser, compare_bcrypt, createJWT, decodeString, deleteAllLogsInObject, deleteLogEntries, deleteUserLogEntries, elapsedSinceRequestStart, encodeString, extractToken, forgotUser, forgotUserAction, forgotUserReset, getAllLogEntries, getAuthRoutes, getClientId, getCurrentUser, getLogEntries, getLogParams, getRawIp, getSignedIp, getTokens, getUserLogEntries, getUserLogPropertyName, handleEntryRequest, hash_bcrypt, isCurrentUserAdmin, isRateLimitReached, isUserAdmin, loginUser, loginUserAction, measureTiming, parseJWT, randomHash, randomHash_SHA1, rateLimitRequest, readRequestBody, refreshUser, registerUser, registerUserAction, res, resRaw, res_200, res_204, res_400, res_401, res_404, res_429, res_500, sendEmail, sign, sign_sha1, sign_sha256 };
